/* SPDX-License-Identifier: GPL-2.0 */
/******************************************************************************
 *
 * Copyright(c) 2007 - 2016  Realtek Corporation.
 *
 * Contact Information:
 * wlanfae <wlanfae@realtek.com>
 * Realtek Corporation, No. 2, Innovation Road II, Hsinchu Science Park,
 * Hsinchu 300, Taiwan.
 *
 * Larry Finger <Larry.Finger@lwfinger.net>
 *
 *****************************************************************************/
/*RTL8822B PHY Parameters*/
/*
 * [Caution]
 * Since 01/Aug/2015, the commit rules will be simplified.
 * You do not need to fill up the version.h anymore,
 * only the maintenance supervisor fills it before formal release.
 */
#define RELEASE_DATE_8822B 20161103
#define COMMIT_BY_8822B "BB_JOE"
#define RELEASE_VERSION_8822B 67
